/***************************************************
 KittyBot Arduino Example Code
 
 Adapted from example code written by Andy Lindsay at Parallax Inc.
 and Limor Fried/LadyAda at Adafruit Industries by Matthew Matz at
 Parallax Inc. 
 ***************************************************/
 

/*************************************************** 
  This is an example for the Adafruit VS1053 Codec Breakout

  Designed specifically to work with the Adafruit VS1053 Codec Breakout 
  ----> https://www.adafruit.com/products/1381

  Adafruit invests time and resources providing this open source code, 
  please support Adafruit and open-source hardware by purchasing 
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.  
  BSD license, all text above must be included in any redistribution
 ****************************************************/

// include SPI, MP3, Servo and SD libraries
#include <SPI.h>             // We will use the hardware SPI pins: CLK (13), MISO (12), MOSI (11)
#include "Adafruit_VS1053.h"
#include <SD.h>
#include <Servo.h>
 
//Set up Servo Objects
Servo servoRight;                            
Servo servoLeft;

// These are the pins used for the music maker shield
#define SHIELD_RESET  -1      // VS1053 reset pin (unused!)
#define SHIELD_CS      7      // VS1053 chip select pin (output)
#define SHIELD_DCS     6      // VS1053 Data/command select pin (output)

// These are common pins between breakout and shield
#define CARDCS         4     // Card chip select pin

// DREQ should be an Int pin, see http://arduino.cc/en/Reference/attachInterrupt
#define DREQ           3       // VS1053 Data request, ideally an Interrupt pin

// The pin for the PING))) sensor
#define pingPin        A0

// The pins for the servos
#define servoPinL      8
#define servoPinR      9

// establish variables for duration of the ping,
// and the distance result in inches and centimeters:
long duration, ping_inches, ping_cm;

// create shield-example object!
Adafruit_VS1053_FilePlayer musicPlayer = Adafruit_VS1053_FilePlayer(SHIELD_RESET, SHIELD_CS, SHIELD_DCS, DREQ, CARDCS);

void setup() {
  freqOut(2, 2000, 3000);

  // use an unconnected analog input to seed the random number generator
  (analogRead(2));

  // initialise the music player
  musicPlayer.begin();  
  delay(100);
  
  // initialise the SD card
  SD.begin(CARDCS); 
  
  // Set volume for left, right channels. lower numbers == louder volume!
  musicPlayer.setVolume(20,20);

  // DREQ is on an interrupt pin (on uno, #2 or #3) so we 
  // can do background audio playing - not used in this example
  musicPlayer.useInterrupt(VS1053_FILEPLAYER_PIN_INT);  // DREQ int
  
  servoLeft.attach(servoPinL);                      // Attach left signal to pin 13
  servoRight.attach(servoPinR);                     // Attach right signal to pin 12
  
}

void loop() {
  delay(50);
  ping();
  
  drive_forward();
  
  if (ping_cm < 21) {  // is there an obstacle?
  	
  	// Stop moving and dettach to prevent servo chatter
    drive_stop();
    servoLeft.detach();                      
    servoRight.detach();

    delay(50);
  
    musicPlayer.playFullFile("meow.wav");
    delay(4000);
    
  	ping();
  	if (ping_cm < 21) {  // is there still an obstacle?
  	  delay(50);
      musicPlayer.playFullFile("hiss.wav");
      delay(100);
      
      // Attach servos and begin driving
      servoLeft.attach(servoPinL);
      servoRight.attach(servoPinR);
      drive_reverse();
      delay(50);
      
      if(random(10)<5) {
        drive_turnLeft();
      } else {
      	drive_turnRight();
      }
      
      //turn a random amount
      delay(random(300,800));
      drive_stop();
      delay(50);
      
  	} else {
      musicPlayer.playFullFile("purrs.wav");
      delay(50);
      servoLeft.attach(servoPinL);
      servoRight.attach(servoPinR);

  	}
  
  }
  
}

void ping() {
  // The PING))) is triggered by a HIGH pulse of 2 or more microseconds.
  // Give a short LOW pulse beforehand to ensure a clean HIGH pulse:
  pinMode(pingPin, OUTPUT);
  digitalWrite(pingPin, LOW);
  delayMicroseconds(2);
  digitalWrite(pingPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(pingPin, LOW);

  // The same pin is used to read the signal from the PING))): a HIGH
  // pulse whose duration is the time (in microseconds) from the sending
  // of the ping to the reception of its echo off of an object.
  pinMode(pingPin, INPUT);
  duration = pulseIn(pingPin, HIGH);

  // convert the time into a distance
  // The speed of sound is 73.746 microseconds per inch (29 microseconds per centimeter).
  // Divide by 2 because it's a round-trip.
  ping_inches = duration / 74 / 2;
  ping_cm = duration / 29 / 2;
  return;
}


void drive_reverse() {
  servoLeft.writeMicroseconds(1700);         // 1.7 ms -> counterclockwise
  servoRight.writeMicroseconds(1300);        // 1.3 ms -> clockwise
  return;
}

void drive_forward() {
  servoLeft.writeMicroseconds(1300);         // 1.3 ms -> clockwise
  servoRight.writeMicroseconds(1700);        // 1.7 ms -> counterclockwise
  return;
}

void drive_turnLeft() {
  servoLeft.writeMicroseconds(1300);         // 1.3 ms -> clockwise
  servoRight.writeMicroseconds(1300);        // 1.3 ms -> clockwise
  return;
}

void drive_turnRight() {
  servoLeft.writeMicroseconds(1700);         // 1.7 ms -> counterclockwise
  servoRight.writeMicroseconds(1700);        // 1.7 ms -> counterclockwise
  return;
}

void drive_stop() {
  servoLeft.writeMicroseconds(1500);         // 1.5 ms -> still
  servoRight.writeMicroseconds(1500);        // 1.5 ms -> still
  return;
}

void freqOut(int tonePin, long toneDura, long toneFreq) {  // used to play tone on startup
  pinMode(tonePin, OUTPUT);
  toneFreq = 500000L / toneFreq - 10L;
  long cycles = toneDura * 500L / toneFreq;
  for (long idx; idx <= cycles; idx++) {
    digitalWrite(tonePin, HIGH);
    delayMicroseconds(toneFreq);
    digitalWrite(tonePin, LOW);
    delayMicroseconds(toneFreq);
  }
}